#ifndef RTOS_H_INCLUDED
#define RTOS_H_INCLUDED

/*
 ************************************************************************
 *                                                                      *
 * RTOS.H : include file for RTOS                                       *
 *                                                                      *
 * to be included in every C-module accessing RTOS-routines             *
 *                                                                      *
 ************************************************************************

 ************************************************************************
 *                                                                      *
 *      Part of embOS real-time operating system                        *
 *                                                                      *
 *              (c) Copyright SEGGER Microcontrollersysteme GmbH        *
 *                                                                      *
 *                                                                      *
 ************************************************************************

*/

/********************************************************
*
*          Chip specifics
*
*********************************************************
*/

#include <In78400.h>      /* intrinsics */
#include <string.h>       /* memset and friends */

#define OS_CPU "NEC K4/IAR"
#define OS_CPU_VERSION 30603

#define OS_DI() _DI();
#define OS_EI() _EI();

#define OS_SP_ADJUST 0
#define OS_GetTime()   ((int)OS_Time)
#define OS_SADDR       saddr    /* OS-data needs to be in short memory */
#define OS_STACKPTR             /* Stacks may be located in whole memory  */

/* Option to supply more info  to viewer */
#define OS_SUPPORT_SYSSTACK_INFO (1)

/********************************************************
*
*          Configuration check
*
*********************************************************
*/
#if (__TID__ & 0x0F) == 0
  #define OS_MODEL "S"   /* Small (Near) model */
  #ifdef OS_FDATA
    #error Not allowed in small model
  #endif
#elif  (__TID__ & 0x0F) == 2
  #define OS_MODEL "L"   /* Large (Far) model */
  #ifndef OS_FDATA
    #define OS_FDATA
  #endif
#else
  #error Unknown memory model !
#endif


/********************************************************************
*
*               Generic portion of embOS starts here
*
*********************************************************************
This file (original name OS_RAW.h) is part of RTOS.h, the include
file for embOS.
*/

#define __EMBOS__ 1               /* Make it possible for application to find out */
#define   EMBOS   1               /* that embOS is compiled and linked */

#define OS_VERSION (30604)



/*********************************************************************
*
*                      Defines for library types
*
**********************************************************************
*/

#if defined(OS_LIBMODE_R)
  #define OS_CHECKSTACK 0
  #define OS_DEBUG 0
  #define OS_PROFILE 0
  #define OS_SUPPORT_TICKSTEP 0
  #define OS_TRACE 0
  #define OS_CreateTask OS_CreateTask_R
  #define OS_LIBMODE "R"
#elif defined(OS_LIBMODE_RP)    /* Not normally generated */
  #define OS_CHECKSTACK 0
  #define OS_DEBUG 0
  #define OS_PROFILE 1
  #define OS_SUPPORT_TICKSTEP 1
  #define OS_TRACE 0
  #define OS_CreateTask OS_CreateTask_RP
  #define OS_LIBMODE "RP"
#elif defined(OS_LIBMODE_S)
  #define OS_CHECKSTACK 1
  #define OS_DEBUG 0
  #define OS_PROFILE 0
  #define OS_SUPPORT_TICKSTEP 0
  #define OS_TRACE 0
  #define OS_CreateTask OS_CreateTask_S
  #define OS_LIBMODE "S"
#elif defined(OS_LIBMODE_SP)
  #define OS_PROFILE 1
  #define OS_CHECKSTACK 1
  #define OS_DEBUG 0
  #define OS_SUPPORT_TICKSTEP 1
  #define OS_TRACE 0
  #define OS_CreateTask OS_CreateTask_SP
  #define OS_LIBMODE "SP"
#elif defined(OS_LIBMODE_D)
  #define OS_PROFILE 0
  #define OS_CHECKSTACK 1
  #define OS_DEBUG 1
  #define OS_SUPPORT_TICKSTEP 1
  #define OS_TRACE 0
  #define OS_CreateTask OS_CreateTask_D
  #define OS_LIBMODE "D"
#elif defined(OS_LIBMODE_DP)
  #define OS_PROFILE 1
  #define OS_CHECKSTACK 1
  #define OS_DEBUG 1
  #define OS_SUPPORT_TICKSTEP 1
  #define OS_TRACE 0
  #define OS_CreateTask OS_CreateTask_DP
  #define OS_LIBMODE "DP"
#elif defined(OS_LIBMODE_ST)
  #define OS_CHECKSTACK 1
  #define OS_DEBUG 0
  #define OS_PROFILE 1
  #define OS_SUPPORT_TICKSTEP 1
  #define OS_TRACE 1
  #define OS_CreateTask OS_CreateTask_ST
  #define OS_LIBMODE "ST"
#elif defined(OS_LIBMODE_DT)
  #define OS_CHECKSTACK 1
  #define OS_DEBUG 1
  #define OS_PROFILE 1
  #define OS_SUPPORT_TICKSTEP 1
  #define OS_TRACE 1
  #define OS_CreateTask OS_CreateTask_DT
  #define OS_LIBMODE "DT"
#elif defined(OS_LIBMODE_T)
  #error This libmode has been renamed. T -> DT
#else
  #error Please define library type used !
#endif

/*
        ****************************************************
        *                                                  *
        *           OS Compile time switches               *
        *                                                  *
        ****************************************************

These compile time switches can be modified when recompiling
the library. Usually, the values are in OSCHIP.H, which is then
merged with OS_RAW.H to form RTOS.H.
If the values are not defined in OSCHIP.H, the default values
below are used.
*/

/* Make it possible to place const data in RAM via
   define and therefor via command line. This is required
	 for some smaller CPUs in small memory models
   It is defined to be "const" by default, but can be defined to nothing
   by simply adding a -DOS_CONST_DATA on the commandline
*/
#ifndef OS_CONST_DATA
  #define OS_CONST_DATA const  /* Default */
#else
  #undef OS_CONST_DATA
  #define OS_CONST_DATA        /* define blank */
#endif

#ifndef OS_MODEL
  #define OS_MODEL ""
#endif

#ifndef OS_MEMSET
  #define OS_MEMSET(a,v,s) memset(a,v,s)
#endif

#ifndef OS_DUMMY_STATEMENT
  #define OS_DUMMY_STATEMENT()
#endif

#ifndef OS_USEPARA
  #define OS_USEPARA(para) para=para
#endif

#ifndef OS_SUPPORT_TICKSTEP
  #define OS_SUPPORT_TICKSTEP 0
#endif

#ifndef OS_SUPPORT_TIMEOUT
  #define OS_SUPPORT_TIMEOUT 1
#endif

#ifndef OS_SUPPORT_DIAG
  #ifdef __OS_SUPPORT_DIAG__
    #define OS_SUPPORT_DIAG 1
  #else
    #define OS_SUPPORT_DIAG 0
  #endif
#endif

#ifndef OS_TRACKNAME
  #define OS_TRACKNAME (1)
#endif

#ifndef OS_TRACE
  #define OS_TRACE (0)
#endif

#ifndef __OS_RRSUPPORTED__
  #define __OS_RRSUPPORTED__ (1)
#endif

#ifndef __OS_CREATETASK0__
  #define  __OS_CREATETASK0__
#endif

#ifndef __OS_STACK_AT_BOTTOM__
  #define __OS_STACK_AT_BOTTOM__ (0)
#endif

#ifndef __OS_STACK_BOTTOM_USED
  #define __OS_STACK_BOTTOM_USED (0)
#endif

/*  Settings for NON-ANSI compiler syntax
    The default behaviour complies with IAR.
*/
#ifndef OS_COMPILER_LOCATION_LEFT
  #define OS_COMPILER_LOCATION_LEFT 1  /* (req. for IAR) */
#endif

/*********************************************************************
*
*                      Config switch checking
*
**********************************************************************

*/

/* Not required. */

/*********************************************************************
*
*                      Define basic types
*
**********************************************************************
*/


#ifndef OS_I8
  #define OS_I8 signed char
#endif  

#ifndef OS_U8
  #define OS_U8 unsigned char
#endif

#ifndef OS_I16
  #define OS_I16 signed short
#endif

#ifndef OS_U16
  #define OS_U16 unsigned short
#endif

#ifndef OS_I32
  #define OS_I32 long
#endif

#ifndef OS_U32
  #define OS_U32 unsigned OS_I32
#endif

/* Defines a true integer. This type is guaranteed
   a) to have at least 8 bits,
   b) to compile and execute best on the target CPU
   It will normally be the same as an int, but on most
   8-bit CPUs it will be replaced by a character because
   the CPU can deal with 8 bits more efficient than with
   16 bit integers.
   Use with care !
*/
#ifndef OS_INT
  #define OS_INT       int
#endif

#ifndef OS_UINT
  #define OS_UINT       unsigned OS_INT
#endif


/*********************************************************************
*
*                      Error codes
*
**********************************************************************
*/

/* stack overflow or invalid stack */
#define OS_ERR_STACK                          (120)
/* invalid or non-initialized data structures */
#define OS_ERR_INV_TASK                       (128)
#define OS_ERR_INV_TIMER                      (129)
#define OS_ERR_INV_MAILBOX                    (130)
#define OS_ERR_INV_CSEMA                      (132)
#define OS_ERR_INV_RSEMA                      (133)

/* Using GetMail1 or PutMail1 or GetMailCond1 or PutMailCond1 on
   a non-1 byte mailbox  */
#define OS_ERR_MAILBOX_NOT1                   (135)

/*internal errors, please contact SEGGER Microcontrollersysteme */
#define OS_ERR_MAILBOX_NOT_IN_LIST            (140)
#define OS_ERR_TASKLIST_CORRUPT               (142)

/* not matching routine calls or macro usage */
#define OS_ERR_UNUSE_BEFORE_USE               (150)
#define OS_ERR_LEAVEREGION_BEFORE_ENTERREGION (151)
#define OS_ERR_LEAVEINT                       (152)
#define OS_ERR_DICNT                          (153)
#define OS_ERR_INTERRUPT_DISABLED             (154)

/* not a legal system call during interrupt */
#define OS_ERR_ILLEGAL_IN_ISR                 (160)
/* not a legal system call during timer */
#define OS_ERR_ILLEGAL_IN_TIMER               (161)

/* double used data structures */
#define OS_ERR_2USE_TASK                      (170)
#define OS_ERR_2USE_TIMER                     (171)
#define OS_ERR_2USE_MAILBOX                   (172)
#define OS_ERR_2USE_BSEMA                     (173)
#define OS_ERR_2USE_CSEMA                     (174)
#define OS_ERR_2USE_RSEMA                     (175)

/* Communication errors (OSCOM) */
#define OS_ERR_NESTED_RX_INT                  (180)


/********************************************************************
*
*                  Trace identifier
*
*********************************************************************

Identifier from 0 to 99 are reserved for embOS
Even when not all of those are currently used,
they may be defined in the future

*/

#define OS_TRACE_ID_DEACTIVATE      (1)
#define OS_TRACE_ID_ACTIVATE        (2)
#define OS_TRACE_ID_TIMERCALLBACK   (3)

/* Task group */
#define OS_TRACE_ID_DELAY           (10)
#define OS_TRACE_ID_DELAYUNTIL      (11)
#define OS_TRACE_ID_SETPRIORITY     (12)
#define OS_TRACE_ID_WAKETASK        (13)
#define OS_TRACE_ID_CREATETASK      (14)
#define OS_TRACE_ID_TERMINATETASK   (15)

/* Event group */
#define OS_TRACE_ID_WAITEVENT       (20)
#define OS_TRACE_ID_WAITEVENT_TIMED (21)
#define OS_TRACE_ID_SIGNALEVENT     (22)
#define OS_TRACE_ID_CLEAREVENTS     (23)

/* Mailbox group  */
#define OS_TRACE_ID_CREATEMB        (30)
#define OS_TRACE_ID_CLEARMB         (31)
#define OS_TRACE_ID_DELETEMB        (32)
#define OS_TRACE_ID_PUTMAIL         (33)
#define OS_TRACE_ID_GETMAIL         (34)
#define OS_TRACE_ID_PUTMAILCOND     (35)
#define OS_TRACE_ID_GETMAILCOND     (36)
#define OS_TRACE_ID_PUTMAIL_TIMED   (37)
#define OS_TRACE_ID_GETMAIL_TIMED   (38)

#define OS_TRACE_ID_PUTMAIL1        (40)
#define OS_TRACE_ID_GETMAIL1        (41)
#define OS_TRACE_ID_PUTMAILCOND1    (42)
#define OS_TRACE_ID_GETMAILCOND1    (43)
#define OS_TRACE_ID_PUTMAIL_TIMED1  (44)
#define OS_TRACE_ID_GETMAIL_TIMED1  (45)

/* Resource group */
#define OS_TRACE_ID_CREATERSEMA     (50)
#define OS_TRACE_ID_USE             (51)
#define OS_TRACE_ID_UNUSE           (52)
#define OS_TRACE_ID_REQUEST         (53)
#define OS_TRACE_ID_GETSEMAVALUE    (54)

/* Counting Semaphore group */
#define OS_TRACE_ID_CREATECSEMA     (60)
#define OS_TRACE_ID_DELETECSEMA     (61)
#define OS_TRACE_ID_SIGNALCSEMA     (62)
#define OS_TRACE_ID_WAITCSEMA       (63)
#define OS_TRACE_ID_WAITCSEMATIMED  (64)

/* Timer group    */
#define OS_TRACE_ID_CREATETIMER     (70)
#define OS_TRACE_ID_DELETETIMER     (71)
#define OS_TRACE_ID_STARTTIMER      (72)
#define OS_TRACE_ID_STOPTIMER       (73)
#define OS_TRACE_ID_RETRIGGERTIMER  (74)
#define OS_TRACE_ID_SETTIMERPERIOD  (75)



/*********************************************************************
*
*                      Assertions
*
**********************************************************************

  Assertions are used to generate code in the debug version
  of embOS in order catch programming faults like
  bad pointers or uninitialized data structures
*/

#if OS_DEBUG
  #define OS_ASSERT(Exp, ErrCode) { if (!(Exp)) OS_Error(ErrCode); }
#else
  #define OS_ASSERT(Exp, ErrCode)
#endif

#define OS_ASSERT_NOINT()     OS_ASSERT((OS_InInt ==0),  OS_ERR_ILLEGAL_IN_ISR)
#define OS_ASSERT_NOTIMER()   OS_ASSERT((OS_InTimer==0), OS_ERR_ILLEGAL_IN_TIMER)
#define OS_ASSERT_DICnt()     OS_ASSERT(((OS_DICnt&0xf0)==0) , OS_ERR_DICNT)


/*********************************************************************
*
*                      Memory attributes
*
**********************************************************************
*/

#ifndef OS_STACKPTR
  #define OS_STACKPTR
#endif

#ifndef OS_SADDR
  #ifdef SADDR
    #define OS_SADDR SADDR
  #else
    #define OS_SADDR
  #endif
#endif

#ifndef OS_DEFFUNC
  #define OS_DEFFUNC
#endif

#ifndef OS_ASM_FUNC 
  #define OS_ASM_FUNC 
#endif

#ifndef OS_DEFPTR
  #define OS_DEFPTR
#endif

/* pointer modifier for data items */
#ifndef OS_DATAPTR
  #define OS_DATAPTR OS_DEFPTR
#endif

#ifndef OS_RSEMAPTR
  #define OS_RSEMAPTR OS_DEFPTR
#endif

#ifndef OS_CSEMAPTR
  #define OS_CSEMAPTR OS_DEFPTR
#endif

#ifndef OS_MBPTR
  #define OS_MBPTR OS_DEFPTR
#endif

#ifndef OS_TCBPTR
  #define OS_TCBPTR OS_DEFPTR
#endif

#ifndef OS_TIMERPTR
  #define OS_TIMERPTR OS_DEFPTR
#endif

#if OS_SUPPORT_DIAG
  void OS_MarkCall(void);
  #define OS_MARKCALL() OS_MarkCall(),
#else
  #define OS_MARKCALL()
#endif


#ifdef __cplusplus
  extern "C" {
#endif




/*
         ********************************************************
         *                                                      *
         *       Data structures                                *
         *                                                      *
         ********************************************************
*/

typedef void voidRoutine(void);
typedef void OS_RX_CALLBACK(OS_U8 Data);
typedef void OS_TIMERROUTINE(void);

typedef struct {
/* Start of assembly relevant section */
  void  OS_DEFPTR   *pNext;          /*  Off = 0/0/0 */
  OS_U8    OS_STACKPTR *pStack;	      /*  Off = 2/4/4 */
/* End of assembly relevant section */
  union {
    int v;
    void     OS_DATAPTR * p;
  } Data;                            /*  Off = 4/6/8 */
  OS_U8 Priority;                       /*  Off = 6/10/12 */
  OS_U8 Stat;			      /*  Off = 7/11/13 */
  #if __OS_RRSUPPORTED__
    OS_U8  TimeSliceRem;
    OS_U8  TimeSliceReload;
  #endif
  #if OS_TRACKNAME
    const char OS_DEFPTR * Name;
  #endif
  void  OS_DEFPTR * pPrev;
  #if OS_SUPPORT_TIMEOUT
    int   Timeout;
  #endif  
  #if OS_CHECKSTACK
    OS_UINT  StackSize;
    #if !__OS_STACK_AT_BOTTOM__
      OS_U8 OS_STACKPTR* pStackBot;
    #endif
  #endif
  #if OS_PROFILE
    OS_U32 ExecTotal;              /* Time spent executing */ 
    OS_U32 ExecLast;               /* Time spent executing (Reference) */ 
    OS_U32 NumActivations;
    OS_U32 NumPreemptions;
    OS_U32 Load;
    #if OS_PROFILE >= 2
      OS_U32 ReactionTimeSys_TS;           /* Time stamp for reaction measurement */
      OS_U32 ReactionTimeSys_Cycles;
      OS_U32 ReactionTimeSysMax_Cycles;
    #endif
  #endif
  #if OS_SUPPORT_DIAG
    voidRoutine* LastCall;
  #endif
  OS_U8  Events;
  #if OS_DEBUG
    char Id;
  #endif
  #ifdef OS_TCB_CPU_EX
    OS_TCB_CPU_EX
  #endif
} OS_TASK;

typedef struct {
  void OS_DEFPTR * pNext;
  voidRoutine* Hook;
  int   Time;
  int   Period;
  char  Active;
#if OS_DEBUG
  char Id;
#endif
} OS_TIMER;

typedef struct {
  OS_TASK OS_DEFPTR * pTask;
  OS_U8 UseCnt;
  OS_U8 WaitCnt;
#if OS_DEBUG
  void * pNext;
  char Id;
#endif
} OS_RSEMA;

typedef struct {
  OS_U8 Cnt;
  OS_U8 WaitCnt;
#if OS_DEBUG
  void OS_DEFPTR* pNext;
  char Id;
#endif
} OS_CSEMA;

typedef struct {
/* void *pNext; */
  char *pData;
  OS_UINT nofMsg;
  OS_UINT maxMsg;
  OS_UINT iRd;
  OS_U8  sizeofMsg;
  OS_U8  WaitCntNE;
  OS_U8  WaitCntNF;
#if OS_DEBUG
  char Id;
#endif
} OS_MAILBOX;

typedef struct {
  OS_U32  Time;
  void    *pCurrentTask;
  void    *p;
  OS_U32  v;
  OS_U8   iRout;
} OS_TRACE_ENTRY;



/*********************************************************************
*
*                      Globals
*
**********************************************************************
*/


/*
    Handle DEFinition (storage is actually allocted) versus DEClaration
    (reference) of RTOS variables depending upon who includes this header file.
*/
#ifdef __OSGLOBAL_C__
  #define OS_EXTERN        /* Declare variables if included by RTOSKERN.C */
#else
  #define OS_EXTERN extern /* Define variables if included by anyone else */
#endif


/*** Start of memory pragma areas ***
 Some compilers can not deal with memory attributes and need pragmas
*/
#if defined(__ghs) && defined(__V850)
  #pragma ghs startzda
#endif

OS_EXTERN OS_SADDR  volatile OS_I32 OS_Time;
OS_EXTERN OS_SADDR  int             OS_TimeDex;
#ifndef WIN32
  OS_EXTERN OS_SADDR  OS_U8 OS_RegionCnt;
  OS_EXTERN OS_SADDR  OS_U8 OS_DICnt;
#else
  #define OS_RegionCnt OS_pCurrentTask->RegionCnt
  #define OS_DICnt     OS_pCurrentTask->DICnt
#endif
OS_EXTERN OS_SADDR  OS_U8  OS_IntMSInc;
OS_EXTERN OS_SADDR  OS_U8  OS_TimeSlice;
OS_EXTERN OS_SADDR  OS_U8  OS_TimeSliceAtStart;
OS_EXTERN OS_SADDR  OS_U8  OS_Pending;
OS_EXTERN OS_SADDR  volatile OS_U8  OS_Status;
#if OS_SUPPORT_TICKSTEP
  OS_EXTERN OS_SADDR  volatile OS_U8   OS_TickStep;
  OS_EXTERN OS_SADDR  volatile int     OS_TickStepTime;
#endif
#ifdef __RTOS_ALLOWFRACT__
  OS_EXTERN OS_SADDR        OS_UINT OS_TicksPerMS;
  OS_EXTERN OS_SADDR        int     OS_IntTicksInc;
#endif
#if OS_PROFILE
  OS_EXTERN OS_SADDR OS_U32 OS_TS_ExecStart;
  OS_EXTERN OS_SADDR OS_U8  ProfilingOn;
#endif
#if OS_DEBUG
  OS_EXTERN OS_SADDR int  OS_TaskSwitchFromIntCnt;
  OS_EXTERN OS_SADDR OS_U8 OS_InInt;
  OS_EXTERN OS_SADDR OS_U8 OS_InTimer;
#endif
extern volatile char OS_TxActive;
#define OS_COM_ClearTxActive() OS_TxActive =0

/*** Pointers ***
  Unfortunately there is no real standard on how to define/declare
  pointers properly with extended memory attributes. There are
  currently 2 different standards, which we both support.
*/
#if OS_COMPILER_LOCATION_LEFT
  OS_EXTERN OS_SADDR  OS_TASK OS_DEFPTR  * OS_pTask;                /* linked list of all Tasks */
  OS_EXTERN OS_SADDR  OS_TASK OS_DEFPTR  * OS_pCurrentTask;          /* pointer to current Task */
  OS_EXTERN OS_SADDR  OS_TASK volatile OS_DEFPTR * volatile OS_pActiveTask;
  OS_EXTERN OS_SADDR  OS_TIMER   OS_DEFPTR * OS_pTimer;               /* linked list of all active Timers */
  OS_EXTERN OS_SADDR  OS_RSEMA   OS_DEFPTR * OS_pRSema;
  OS_EXTERN OS_SADDR  OS_MAILBOX OS_DEFPTR * OS_pMB;
  #if OS_DEBUG
    OS_EXTERN OS_SADDR OS_TASK OS_DEFPTR  * OS_pTask2Debug;                /* linked list of all Tasks */
    OS_EXTERN OS_SADDR voidRoutine        * OS_pfOnTask2Debug;
    extern         OS_CSEMA OS_CSEMAPTR* pCSema;
  #endif
#else
  OS_EXTERN OS_TASK    OS_DEFPTR          * OS_SADDR OS_pTask;                /* linked list of all Tasks */
  OS_EXTERN OS_TASK    OS_DEFPTR          * OS_SADDR OS_pCurrentTask;         /* pointer to current Task */
  OS_EXTERN OS_TASK    OS_DEFPTR volatile * OS_SADDR volatile OS_pActiveTask;
  OS_EXTERN OS_TIMER   OS_DEFPTR          * OS_SADDR OS_pTimer;       /* linked list of all active Timers */
  OS_EXTERN OS_RSEMA   OS_DEFPTR          * OS_SADDR OS_pRSema;
  OS_EXTERN OS_MAILBOX OS_DEFPTR          * OS_SADDR OS_pMB;
  #if OS_DEBUG
    OS_EXTERN OS_TASK     OS_DEFPTR * OS_SADDR OS_pTask2Debug;                /* linked list of all Tasks */
    OS_EXTERN voidRoutine           * OS_SADDR OS_pfOnTask2Debug;
    extern   OS_CSEMA OS_CSEMAPTR* pCSema;
  #endif
#endif

/*** Start of memory pragma areas ****/
#if defined(__ghs) && defined(__V850)
  #pragma ghs endzda
#endif

/*
   *********************************************
   *                                           *
   *          Create Task routine and macros   *
   *                                           *
   *********************************************
*/
#if OS_TRACKNAME
  #define CTPARA_NAME "unnamed task",
#else
  #define CTPARA_NAME
#endif

#if __OS_RRSUPPORTED__
  #define CTPARA_TIMESLICE ,2
#else
  #define CTPARA_TIMESLICE 
#endif

/*
   This is the old form. For compatibility reasons it is
   still supported.
*/
#define CREATETASK(pTask, Hook, Priority, pStack) \
OS_CreateTask (pTask,          \
               CTPARA_NAME    \
               Priority,       \
               Hook,           \
               (void OS_STACKPTR*)pStack,  \
               sizeof(pStack)  \
               CTPARA_TIMESLICE \
               )

#if OS_TRACKNAME
  #define OS_CREATETASK(pTask, Name, Hook, Priority, pStack) \
  OS_CreateTask (pTask,          \
                  Name,           \
                  Priority,       \
                  Hook,           \
                  (void OS_STACKPTR*)pStack,  \
                  sizeof(pStack)  \
                  CTPARA_TIMESLICE \
               )
#else
  #define OS_CREATETASK(pTask, Name, Hook, Priority, pStack) \
  OS_CreateTask (pTask,          \
                  Priority,       \
                  Hook,           \
                  (void OS_STACKPTR*)pStack,  \
                  sizeof(pStack)  \
                  CTPARA_TIMESLICE \
               )
#endif



OS_DEFFUNC void OS_CreateTask(
        OS_TASK OS_DEFPTR * pt,
#if OS_TRACKNAME
        const char*Name,
#endif
        OS_U8 Priority,
        voidRoutine*pRoutine,
        void OS_STACKPTR *pStack,
        OS_UINT StackSize
#if __OS_RRSUPPORTED__
        ,OS_UINT TimeSlice
#endif
        );

/*********************************************************************
*
*                      Task routines
*
**********************************************************************
*/

OS_DEFFUNC void OS__Delay(int ms);
OS_DEFFUNC void OS__Delay_Trace(int ms);
OS_DEFFUNC void OS__DelayUntil(int t0);
OS_DEFFUNC void OS__DelayUntil_Trace(int t0);
OS_DEFFUNC void OS__Terminate         (OS_TASK OS_DEFPTR *);
OS_DEFFUNC void OS__Terminate_Trace   (OS_TASK OS_DEFPTR *);
OS_DEFFUNC void OS__WakeTask          (OS_TASK OS_DEFPTR *);
OS_DEFFUNC void OS__WakeTask_Trace    (OS_TASK OS_DEFPTR *);
OS_DEFFUNC void OS_GetState           (OS_TASK OS_DEFPTR *);
OS_DEFFUNC void OS__SetPriority       (OS_TASK OS_DEFPTR *ptask, OS_U8 Priority);
OS_DEFFUNC void OS__SetPriority_Trace (OS_TASK OS_DEFPTR *ptask, OS_U8 Priority);
OS_DEFFUNC char OS_IsTask             (OS_TASK OS_DEFPTR *ptask);

#if OS_TRACE
  #define OS_Delay(p0)          OS__Delay_Trace(p0)
  #define OS_DelayUntil(p0)     OS__DelayUntil_Trace(p0)
  #define OS_SetPriority(p0,p1) OS__SetPriority_Trace(p0,p1)
  #define OS_Terminate(p0)      OS__Terminate_Trace(p0)
  #define OS_WakeTask(p0)       OS__WakeTask_Trace(p0)
#else
  #define OS_Delay(p0)          OS__Delay(p0)
  #define OS_DelayUntil(p0)     OS__DelayUntil(p0)
  #define OS_SetPriority(p0,p1) OS__SetPriority(p0,p1)
  #define OS_Terminate(p0)      OS__Terminate(p0)
  #define OS_WakeTask(p0)       OS__WakeTask(p0)
#endif

/*********************************************************************
*
*                      Info routines
*
**********************************************************************
*/

#define OS_TASKID OS_TASK OS_DEFPTR *
#define OS_GetTaskID() OS_pCurrentTask

#if OS_TRACKNAME
  OS_DEFFUNC const char* OS_GetTaskName       (OS_TASK OS_DEFPTR *);
#else
  #define          OS_GetTaskName(pt)  ((char*)NULL)
#endif

#if OS_CHECKSTACK
  OS_DEFFUNC void  OS_CheckStack(void);              /* internal usage */
  OS_DEFFUNC int   OS_GetStackSpace  (OS_TASK OS_DEFPTR *);
  OS_DEFFUNC int   OS_GetStackUsed   (OS_TASK OS_DEFPTR *);
  OS_DEFFUNC int   OS_GetStackSize   (OS_TASK OS_TCBPTR *pt);
  OS_DEFFUNC void OS_STACKPTR* OS_GetStackBase   (OS_TASK OS_TCBPTR *pt);
#else
  #define OS_CheckStack()
  #define OS_GetStackSpace(pt) 0
  #define OS_GetStackUsed(pt)  0
  #define OS_GetStackSize(pt)  0
  #define OS_GetStackBase(pt)  0
#endif

#if __OS_RRSUPPORTED__
  OS_U8 OS_SetTimeSlice(OS_TASK OS_DEFPTR *pTask, OS_U8 TimeSlice);
  OS_DEFFUNC OS_U8 OS_GetTimeSliceRem(OS_TASK OS_TCBPTR *pTask);
  #define OS_GetTimeSliceReload(pTask)  pTask->TimeSliceReload
#else
  #define OS_GetTimeSliceRem(pTask)     0
  #define OS_GetTimeSliceReload(pTask)  0
#endif

#if OS_DEBUG
  OS_DEFFUNC OS_U8    OS_GetPriority       (OS_TASK OS_DEFPTR *ptask);
#else
  #define OS_GetPriority(pt) pt->Priority 
#endif

OS_DEFFUNC int   OS_GetNumTasks       (void);
OS_DEFFUNC OS_TASK OS_DEFPTR * OS_TaskIndex2Ptr     (int TaskIndex);

#if OS_PROFILE >=1
  OS_DEFFUNC void OS_STAT_NotifyExecEnd(void);
  OS_DEFFUNC OS_U32 OS_STAT_GetExecTime_Cycles (OS_TASK OS_DEFPTR *pTask);
  OS_DEFFUNC int    OS_STAT_GetLoad            (OS_TASK OS_DEFPTR *pTask);
  OS_DEFFUNC OS_U32 OS_STAT_GetNumActivations  (OS_TASK OS_DEFPTR *pTask);
  OS_DEFFUNC OS_U32 OS_STAT_GetNumPreemptions  (OS_TASK OS_DEFPTR *pTask);
  OS_DEFFUNC void   OS_STAT_NotifyReadyStart   (OS_TASK OS_DEFPTR *pTask);
  OS_DEFFUNC void   OS_STAT_Sample(void);
  OS_DEFFUNC void   OS_STAT_SetClocksPerUnit(unsigned int ClocksPerUnit);
#else
  #define OS_STAT_NotifyExecEnd()
  #define OS_STAT_GetExecTime_Cycles(pTask)   0
  #define OS_STAT_GetLoad(pTask)              0
  #define OS_STAT_GetNumActivations(pTask)    0
  #define OS_STAT_GetNumPreemptions(pTask)    0
  #define OS_STAT_Sample()
  #define OS_STAT_SetClocksPerUnit(ClocksPerUnit)
  #define OS_STAT_NOTIFYREADYSTART(pTask)
#endif

#if OS_PROFILE >=2
  #define OS_STAT_NOTIFYREADYSTART(pTask) OS_STAT_NotifyReadyStart(pTask)
#else
  #define OS_STAT_NOTIFYREADYSTART(pTask)
#endif

/*********************************************************************
*
*                      Critical regions
*
**********************************************************************
*/

#define OS_EnterRegion() {OS_RegionCnt++; }
OS_DEFFUNC void OS_LeaveRegion(void);

/* Macro for OS_Leaveregion.
   Main purposes:
   - Used in OS-Kernel
   - Offers the high speed variant (macro) instead of the function call
*/

#define OS_LEAVEREGION_STD() \
 OS_DI(); \
 if (!--OS_RegionCnt) { \
   if (OS_Pending) OS_Switch(); \
  } OS_RestoreI();

#if OS_DEBUG
 #define OS_LEAVEREGION() \
   if (!OS_RegionCnt) OS_Error(-1); \
   OS_LEAVEREGION_STD()
#else
 #define OS_LEAVEREGION()  OS_LEAVEREGION_STD()
#endif



/*********************************************************************
*
*                      Interrupt save/disable/restore macros
*
**********************************************************************
*/
#define OS_IncDI()       { OS_ASSERT_DICnt(); OS_DI(); OS_DICnt++; }
#define OS_DecRI()       { OS_ASSERT_DICnt(); if (--OS_DICnt==0) OS_EI(); }
#define OS_RestoreI()    { OS_ASSERT_DICnt(); if (OS_DICnt==0)   OS_EI(); }



/*********************************************************************
*
*                      ISR (Interrupt service routine) support
*
**********************************************************************
*/
#if OS_DEBUG
  #define OS_MARK_IN_ISR()         {OS_InInt++;}
  #define OS_MARK_OUTOF_ISR()      {if (!OS_InInt--) OS_Error(OS_ERR_LEAVEINT);}
#else
  #define OS_MARK_IN_ISR()
  #define OS_MARK_OUTOF_ISR()
#endif


#ifndef OS_EnterInterrupt   /* Allow override in OSChip.h */
  #define OS_EnterInterrupt() { \
    OS_MARK_IN_ISR(); \
    OS_RegionCnt++; \
    OS_DICnt++; \
  }
#endif

#ifndef OS_EnterNestableInterrupt   /* Allow override in OSChip.h */
  #define OS_EnterNestableInterrupt() { \
    OS_MARK_IN_ISR();  \
    OS_RegionCnt++; \
    OS_EI(); \
  }
#endif

#ifndef OS_LeaveInterrupt   /* Allow override in OSChip.h */
  #define OS_LeaveInterrupt() { \
    OS_MARK_OUTOF_ISR(); \
    OS_DICnt--; /* Must have been zero initially ! (We could put =0 instead) */\
    if (!--OS_RegionCnt) { \
      if (OS_Pending) { \
        OS_SwitchFromInt(); \
      } \
    } \
  }
#endif

#ifndef OS_LeaveNestableInterrupt   /* Allow override in OSChip.h */
  #define OS_LeaveNestableInterrupt() { \
    OS_MARK_OUTOF_ISR();       \
    OS_DI();                \
    if (!--OS_RegionCnt) {  \
      if (OS_Pending) {     \
        OS_SwitchFromInt(); \
      }                     \
    }                       \
  }
#endif

#ifndef OS_LeaveInterruptNoSwitch   /* Allow override in OSChip.h */
  #define OS_LeaveInterruptNoSwitch() { \
    OS_MARK_OUTOF_ISR(); \
    OS_DICnt--; \
    OS_RegionCnt--; \
  }
#endif

#ifndef OS_LeaveNestableInterruptNoSwitch   /* Allow override in OSChip.h */
  #define OS_LeaveNestableInterruptNoSwitch() { \
    OS_MARK_OUTOF_ISR(); \
    OS_DI(); \
    OS_RegionCnt--; \
  }
#endif

#ifndef OS_EnterIntStack
  OS_DEFFUNC void OS__EnterIntStack(void);
  #define OS_EnterIntStack() {OS_DI(); OS__EnterIntStack(); OS_RestoreI(); }
#endif

#ifndef OS_LeaveIntStack
  OS_DEFFUNC void OS__LeaveIntStack(void);
  #define OS_LeaveIntStack() {OS_DI(); OS__LeaveIntStack(); }
#endif

/*********************************************************************
*
*                      Resource semaphores
*
**********************************************************************
*/

OS_DEFFUNC void     OS__Use             (OS_RSEMA OS_DEFPTR *);
OS_DEFFUNC void     OS__Use_Trace       (OS_RSEMA OS_DEFPTR *);
OS_DEFFUNC void     OS__Unuse           (OS_RSEMA OS_DEFPTR *);
OS_DEFFUNC void     OS__Unuse_Trace     (OS_RSEMA OS_DEFPTR *);
OS_DEFFUNC char     OS__Request         (OS_RSEMA OS_DEFPTR *);
OS_DEFFUNC char     OS__Request_Trace   (OS_RSEMA OS_DEFPTR *);
OS_DEFPTR  int      OS_GetSemaValue     (OS_RSEMA OS_DEFPTR *);   /* get current value of semaphore */
OS_DEFFUNC OS_TASK* OS_GetResourceOwner (OS_RSEMA OS_DEFPTR *);   /* get TaskID of task using semaphore */

OS_DEFFUNC void     OS__CreateRSema       (OS_RSEMA OS_DEFPTR *);
OS_DEFFUNC void     OS__CreateRSema_Trace (OS_RSEMA OS_DEFPTR *);

#define OS_CREATERSEMA(ps) OS_CreateRSema(ps)

#if OS_TRACE
  #define OS_CreateRSema(p0) OS__CreateRSema_Trace(p0)
  #define OS_Use(p0)         OS__Use_Trace(p0)
  #define OS_Unuse(p0)       OS__Unuse_Trace(p0)
  #define OS_Request(p0)     OS__Request_Trace(p0)
#else
  #define OS_CreateRSema(p0) OS__CreateRSema(p0)
  #define OS_Use(p0)         OS__Use(p0)
  #define OS_Unuse(p0)       OS__Unuse(p0)
  #define OS_Request(p0)     OS__Request(p0)
#endif


/*********************************************************************
*
*                      Counting semaphores
*
**********************************************************************
*/


OS_DEFFUNC void OS__CreateCSema           (OS_CSEMA OS_DEFPTR *, OS_U8 InitValue);
OS_DEFFUNC void OS__CreateCSema_Trace     (OS_CSEMA OS_DEFPTR *, OS_U8 InitValue);
OS_DEFFUNC void OS__DeleteCSema           (OS_CSEMA OS_DEFPTR *);
OS_DEFFUNC void OS__DeleteCSema_Trace     (OS_CSEMA OS_DEFPTR *);
OS_DEFPTR  int  OS_GetCSemaValue          (OS_CSEMA OS_DEFPTR *);
OS_DEFFUNC void OS__SignalCSema           (OS_CSEMA OS_DEFPTR *);
OS_DEFFUNC void OS__SignalCSema_Trace     (OS_CSEMA OS_DEFPTR *);
OS_DEFFUNC void OS__WaitCSema             (OS_CSEMA OS_DEFPTR *);
OS_DEFFUNC void OS__WaitCSema_Trace       (OS_CSEMA OS_DEFPTR *);
OS_DEFFUNC int  OS__WaitCSemaTimed        (OS_CSEMA OS_DEFPTR *pSema, int TimeOut);
OS_DEFFUNC int  OS__WaitCSemaTimed_Trace  (OS_CSEMA OS_DEFPTR *pSema, int TimeOut);
#define OS_CREATECSEMA(ps) OS_CreateCSema(ps,0)

#if OS_TRACE
  #define OS_CreateCSema(p0, p1)     OS__CreateCSema_Trace(p0, p1)
  #define OS_DeleteCSema(p0)         OS__DeleteCSema_Trace(p0)
  #define OS_SignalCSema(p0)         OS__SignalCSema_Trace(p0)
  #define OS_WaitCSema(p0)           OS__WaitCSema_Trace(p0)
  #define OS_WaitCSemaTimed(p0, p1)  OS__WaitCSemaTimed_Trace(p0, p1)
#else
  #define OS_CreateCSema(p0, p1)     OS__CreateCSema(p0, p1)
  #define OS_DeleteCSema(p0)         OS__DeleteCSema(p0)
  #define OS_SignalCSema(p0)         OS__SignalCSema(p0)
  #define OS_WaitCSema(p0)           OS__WaitCSema(p0)
  #define OS_WaitCSemaTimed(p0, p1)  OS__WaitCSemaTimed(p0, p1)
#endif

/*********************************************************************
*
*                      Mailboxes
*
**********************************************************************
*/
#define CREATEMB(MAILBOX, size, max, Buffer) OS_CreateMB(MAILBOX,size, max, Buffer);
OS_DEFFUNC void OS__CreateMB           (OS_MAILBOX OS_DEFPTR * pMB, OS_U8 sizeofMsg, OS_UINT maxnofMsg, void* Buffer);    /* initialize mailbox */
OS_DEFFUNC void OS__CreateMB_Trace     (OS_MAILBOX OS_DEFPTR * pMB, OS_U8 sizeofMsg, OS_UINT maxnofMsg, void* Buffer);    /* initialize mailbox */
OS_DEFFUNC void OS__ClearMB            (OS_MAILBOX OS_DEFPTR * pMB);
OS_DEFFUNC void OS__ClearMB_Trace      (OS_MAILBOX OS_DEFPTR * pMB);
OS_DEFFUNC void OS__PutMail            (OS_MAILBOX OS_DEFPTR * pMB, void* pmail);
OS_DEFFUNC void OS__PutMail_Trace      (OS_MAILBOX OS_DEFPTR * pMB, void* pmail);
OS_DEFFUNC char OS__PutMailCond        (OS_MAILBOX OS_DEFPTR * pMB, void* pmail);
OS_DEFFUNC char OS__PutMailCond_Trace  (OS_MAILBOX OS_DEFPTR * pMB, void* pmail);
OS_DEFFUNC void OS__GetMail            (OS_MAILBOX OS_DEFPTR * pMB, void* Result);
OS_DEFFUNC void OS__GetMail_Trace      (OS_MAILBOX OS_DEFPTR * pMB, void* Result);
OS_DEFFUNC char OS__GetMailCond        (OS_MAILBOX OS_DEFPTR * pMB, void* Result);
OS_DEFFUNC char OS__GetMailCond_Trace  (OS_MAILBOX OS_DEFPTR * pMB, void* Result);
OS_DEFFUNC void OS__PutMail1           (OS_MAILBOX OS_DEFPTR * pMB, const char* pmail);
OS_DEFFUNC void OS__PutMail1_Trace     (OS_MAILBOX OS_DEFPTR * pMB, const char* pmail);
OS_DEFFUNC char OS__PutMailCond1       (OS_MAILBOX OS_DEFPTR * pMB, const char* pmail);
OS_DEFFUNC char OS__PutMailCond1_Trace (OS_MAILBOX OS_DEFPTR * pMB, const char* pmail);
OS_DEFFUNC void OS__GetMail1           (OS_MAILBOX OS_DEFPTR * pMB, char* Result);
OS_DEFFUNC void OS__GetMail1_Trace     (OS_MAILBOX OS_DEFPTR * pMB, char* Result);
OS_DEFFUNC char OS__GetMailCond1       (OS_MAILBOX OS_DEFPTR * pMB, char* Result);
OS_DEFFUNC char OS__GetMailCond1_Trace (OS_MAILBOX OS_DEFPTR * pMB, char* Result);
OS_DEFFUNC void OS__DeleteMB           (OS_MAILBOX OS_DEFPTR * pThis);
OS_DEFFUNC void OS__DeleteMB_Trace     (OS_MAILBOX OS_DEFPTR * pThis);


#if OS_DEBUG ==0
  #define OS_GetMessageCnt(pMB) (*pMB).nofMsg
#else
  OS_DEFFUNC int  OS_GetMessageCnt(OS_MAILBOX OS_DEFPTR *);   /* get no. of available Messages */
#endif

#if OS_TRACE
	#define OS_CreateMB     OS__CreateMB_Trace
	#define OS_ClearMB      OS__ClearMB_Trace
	#define OS_PutMail      OS__PutMail_Trace
	#define OS_PutMailCond  OS__PutMailCond_Trace
	#define OS_GetMail      OS__GetMail_Trace
	#define OS_GetMailCond  OS__GetMailCond_Trace
	#define OS_PutMail1     OS__PutMail1_Trace
	#define OS_PutMailCond1 OS__PutMailCond1_Trace
	#define OS_GetMail1     OS__GetMail1_Trace
	#define OS_GetMailCond1 OS__GetMailCond1_Trace
	#define OS_DeleteMB     OS__DeleteMB_Trace
#else
	#define OS_CreateMB     OS__CreateMB
	#define OS_ClearMB      OS__ClearMB
	#define OS_PutMail      OS__PutMail
	#define OS_PutMailCond  OS__PutMailCond
	#define OS_GetMail      OS__GetMail
	#define OS_GetMailCond  OS__GetMailCond
	#define OS_PutMail1     OS__PutMail1
	#define OS_PutMailCond1 OS__PutMailCond1
	#define OS_GetMail1     OS__GetMail1
	#define OS_GetMailCond1 OS__GetMailCond1
	#define OS_DeleteMB     OS__DeleteMB
#endif

/*********************************************************************
*
*                      Events
*
**********************************************************************
*/

OS_DEFFUNC char OS__ClearEvents           (OS_TASK OS_DEFPTR *);
OS_DEFFUNC char OS__ClearEvents_Trace     (OS_TASK OS_DEFPTR *);
OS_DEFFUNC char OS__GetEventsOccured      (OS_TASK OS_DEFPTR *);
OS_DEFFUNC void OS__SignalEvent           (char Event, OS_TASK OS_DEFPTR *);
OS_DEFFUNC void OS__SignalEvent_Trace     (char Event, OS_TASK OS_DEFPTR *);
OS_DEFFUNC char OS__WaitEvent             (char EventMask);
OS_DEFFUNC char OS__WaitEvent_Trace       (char EventMask);
OS_DEFFUNC char OS__WaitEventTimed        (char EventMask, int TimeOut);
OS_DEFFUNC char OS__WaitEventTimed_Trace  (char EventMask, int TimeOut);


#define OS_GetEvents          OS_MARKCALL() OS__GetEvents
#define OS_GetEventsOccured   OS_MARKCALL() OS__GetEventsOccured

#if OS_TRACE
  #define OS_ClearEvents        OS_MARKCALL() OS__ClearEvents_Trace
  #define OS_SignalEvent        OS_MARKCALL() OS__SignalEvent_Trace
  #define OS_WaitEvent          OS_MARKCALL() OS__WaitEvent_Trace
  #define OS_WaitEventTimed     OS_MARKCALL() OS__WaitEventTimed_Trace
#else
  #define OS_ClearEvents        OS_MARKCALL() OS__ClearEvents
  #define OS_SignalEvent        OS_MARKCALL() OS__SignalEvent
  #define OS_WaitEvent          OS_MARKCALL() OS__WaitEvent
  #define OS_WaitEventTimed     OS_MARKCALL() OS__WaitEventTimed
#endif

/*********************************************************************
*
*                      Timers
*
**********************************************************************
*/

OS_DEFFUNC void    OS__CreateTimer          (OS_TIMER OS_DEFPTR *, OS_TIMERROUTINE* Callback, OS_UINT Timeout);
OS_DEFFUNC void    OS__CreateTimer_Trace    (OS_TIMER OS_DEFPTR *, OS_TIMERROUTINE* Callback, OS_UINT Timeout);
OS_DEFFUNC void    OS__DeleteTimer          (OS_TIMER OS_DEFPTR *);
OS_DEFFUNC void    OS__DeleteTimer_Trace    (OS_TIMER OS_DEFPTR *);
OS_DEFFUNC int     OS__GetTimerPeriod       (OS_TIMER OS_DEFPTR *);
OS_DEFFUNC OS_U8   OS__GetTimerStatus       (OS_TIMER OS_DEFPTR *);
OS_DEFFUNC int     OS__GetTimerValue        (OS_TIMER OS_DEFPTR *);
OS_DEFFUNC void    OS__SetTimerPeriod       (OS_TIMER OS_DEFPTR *, OS_UINT Period);
OS_DEFFUNC void    OS__SetTimerPeriod_Trace (OS_TIMER OS_DEFPTR *, OS_UINT Period);
OS_DEFFUNC void    OS__StartTimer           (OS_TIMER OS_DEFPTR *);
OS_DEFFUNC void    OS__StartTimer_Trace     (OS_TIMER OS_DEFPTR *);
OS_DEFFUNC void    OS__StopTimer            (OS_TIMER OS_DEFPTR *);
OS_DEFFUNC void    OS__StopTimer_Trace      (OS_TIMER OS_DEFPTR *);
OS_DEFFUNC void    OS__RetriggerTimer       (OS_TIMER OS_DEFPTR *);
OS_DEFFUNC void    OS__RetriggerTimer_Trace (OS_TIMER OS_DEFPTR *);

#define OS_CREATETIMER(pTimer,c,d) \
        OS_CreateTimer(pTimer,c,d); \
        OS_StartTimer(pTimer);

#define OS_GetTimerPeriod OS_MARKCALL() OS__GetTimerPeriod
#define OS_GetTimerStatus OS_MARKCALL() OS__GetTimerStatus
#define OS_GetTimerValue  OS_MARKCALL() OS__GetTimerValue

#if OS_TRACE
  #define OS_CreateTimer    OS_MARKCALL() OS__CreateTimer_Trace
  #define OS_DeleteTimer    OS_MARKCALL() OS__DeleteTimer_Trace
  #define OS_RetriggerTimer OS_MARKCALL() OS__RetriggerTimer_Trace
  #define OS_SetTimerPeriod OS_MARKCALL() OS__SetTimerPeriod_Trace
  #define OS_StartTimer     OS_MARKCALL() OS__StartTimer_Trace
  #define OS_StopTimer      OS_MARKCALL() OS__StopTimer_Trace
#else
  #define OS_CreateTimer    OS_MARKCALL() OS__CreateTimer
  #define OS_DeleteTimer    OS_MARKCALL() OS__DeleteTimer
  #define OS_RetriggerTimer OS_MARKCALL() OS__RetriggerTimer
  #define OS_SetTimerPeriod OS_MARKCALL() OS__SetTimerPeriod
  #define OS_StartTimer     OS_MARKCALL() OS__StartTimer
  #define OS_StopTimer      OS_MARKCALL() OS__StopTimer
#endif

/*********************************************************************
*
*                      Info routines
*
**********************************************************************
*/

OS_DEFFUNC const char* OS_GetCPU(void);
OS_DEFFUNC const char* OS_GetLibMode(void);
OS_DEFFUNC const char* OS_GetModel(void);
OS_DEFFUNC const char* OS_GetLibName(void);
#define OS_GetVersion() ((unsigned int)OS_VERSION)
#define OS_GetType() OS_GetLibMode()

/*********************************************************************
*
*                      Timing support
*
**********************************************************************

 New in V3.04, but not yet documented to the end user. Great for
 run-time measurement.
*/

#define OS_TIMING OS_U32
void    OS_Timing_Start(OS_TIMING* pCycle);
void    OS_Timing_End(OS_TIMING* pCycle);
OS_U32  OS_Timing_Getus(OS_TIMING* pCycle);
#define OS_Timing_GetCycles(pPara) (*pPara)

/*********************************************************************
*
*                      Advanced profiling support
*
**********************************************************************
*/
#if OS_PROFILE >= 1
  void OS_EnableProfiling(int Period);
  #define OS_DisableProfiling() { OS_ProfilingOn=0; }
#else
  #define OS_EnableProfiling(Period);
  #define OS_DisableProfiling()
#endif

/*
   *********************************************
   *                                           *
   *       Configuration                       *
   *                                           *
   *********************************************
*/

#ifdef __RTOS_ALLOWFRACT__  /* For K0 only */
  #define OS_CONFIG(f, div) \
  OS_TicksPerMS  = f/1000; \
  OS_IntMSInc    = div*1000L/f; \
  OS_IntTicksInc =  div-(f/1000)*(div*1000L/f);
#else
  #define OS_CONFIG(f, div) \
  OS_IntMSInc    = div*1000L/f;
#endif

/*********************************************************************
*
*                      Trace support
*
**********************************************************************

Trace support is enabled by defining OS_TRACE 1.
This is automatically done, when OS_LIBMODE_T is defined.

*/

#if OS_TRACE
  void OS_TraceVoid (OS_U8 id);
  void OS_TracePtr  (OS_U8 id, void* p);
  void OS_TraceData (OS_U8 id, int v);
  void OS_TraceDataPtr (OS_U8 id, int v, void*p);
  void OS_TraceU32Ptr  (OS_U8 id, OS_U32 p0, void*p1);
  void OS_TraceEnable(void);
  void OS_TraceDisable(void);
  void OS_TraceEnableAll(void);
  void OS_TraceDisableAll(void);
  void OS_TraceEnableId(OS_U8 id);
  void OS_TraceDisableId(OS_U8 id);
  void OS_TraceEnableFilterId(OS_U8 FilterIndex, OS_U8 id);
  void OS_TraceDisableFilterId(OS_U8 FilterIndex, OS_U8 id);
#else
  #define OS_TraceVoid(id)
  #define OS_TracePtr(id, p)
  #define OS_TraceData(id, v)
  #define OS_TraceDataPtr(id, v, p)
  #define OS_TraceU32Ptr(id, p0, p1)
  #define OS_TraceEnable()
  #define OS_TraceDisable()
  #define OS_TraceEnableAll()
	#define OS_TraceDisableAll()
	#define OS_TraceEnableId(id)
  #define OS_TraceDisableId(id)
  #define OS_TraceEnableFilterId(FilterIndex, id)
  #define OS_TraceDisableFilterId(FilterIndex, id)
#endif  

/*
*********************************************
*
*              RTOS misc.
*
*********************************************
The routines declared below are available on all targets.

*/

/* Routines in RTOSInit.c */
void   OS_InitHW(void);
void   OS_Idle(void);
void   OS_COM_Init(void);
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles);
OS_U32 OS_GetTime_Cycles(void);
void   OS_Error(int code);

/* Routines in library */
OS_ASM_FUNC void OS_Start(void);            /* in RTOS.ASM */
OS_ASM_FUNC void OS_Switch(void);           /* in RTOS.ASM */
OS_ASM_FUNC void OS_SwitchFromInt(void);    /* in RTOS.ASM */

void OS_InitKern(void);         /* in OSKern.c */
char OS_OnTx(void);             /* in OSOnCom.c Returns 1 on last byte */
void OS_OnRx(unsigned char c);  /* in OSOnCom.c */
void OS_Send1(OS_U8 c);
void OS_SendString(const char* s);
OS_RX_CALLBACK* OS_SetRxCallback(OS_RX_CALLBACK* cb);


/*
*********************************************
*
*              RTOS misc. (optional)
*
*********************************************
The routines declared below are available on some targets
only. (Because they do not make much sense on others)
*/

/* Timing routines. Their existance depends on the CPU. In general,
  8-bit CPUs require both routines, where 16-bit CPUs require one
  and 32-bit CPUs require none of these.
*/
#ifndef OS_GetTime
  OS_DEFPTR int OS_GetTime(void);
#endif
#ifndef OS_GetTime32
  OS_DEFPTR OS_I32 OS_GetTime32(void);
#endif

void OS__di(void);
void OS__ei(void);
void OS__SaveIPL(void);
void OS__RestIPL(void);
void OS_SIM_Init(void);

/* Routines for the PC-version running in native mode (non-windows) */
void OS_SetIntFreq(OS_UINT f);
void OS_Exit(int code);
void OS_UseDos(void);
void OS_UnuseDos(void);
int  OS_GetKey(void);
int  OS_WaitKey(void);


/*
   *********************************************
   *                                           *
   *       for compatibility with old version  *
   *                                           *
   *********************************************

Please note: If you do not need compatibility to versions prior to
V2.40, you may erase the following lines !

*/

#ifdef OS_COMPATIBLE_TO_OLDVERSION

#define Delay             OS_Delay
#define DelayUntil        OS_DelayUntil
#define Terminate         OS_Terminate
#define WakeTask          OS_WakeTask
#define GetState          OS_GetState

#define SignalEvent       OS_SignalEvent
#define WaitEvent         OS_WaitEvent
#define GetEventsOccured  OS_GetEventsOccured
#define ClearEvents       OS_ClearEvents

#define StopTimer         OS_StopTimer
#define StartTimer        OS_StartTimer
#define RefreshTimer      OS_RetriggerTimer
#define RetriggerTimer    OS_RetriggerTimer
#define GetTimer          OS_GetTimer
#define CREATETIMER       OS_CREATETIMER


#define SignalCSema      OS_SignalCSema
#define WaitCSema        OS_WaitCSema
#define GetCSemaValue    OS_GetCSemaValue
#define CREATECSEMA      OS_CREATECSEMA

#define Use              OS_Use
#define Unuse            OS_Unuse
#define Request          OS_Request
#define GetResourceOwner OS_ResourceOwner
#define CREATERSEMA      OS_CREATERSEMA
#define GetSemaValue     OS_GetUseCnt
#define OS_GetUseCnt     OS_GetSemaValue


#define EnterInterrupt   OS_EnterInterrupt
#define LeaveInterrupt   OS_LeaveInterrupt
#define EnterIntStack    OS_EnterIntStack
#define LeaveIntStack    OS_LeaveIntStack
#define EnterRegion      OS_EnterRegion
#define LeaveRegion      OS_LeaveRegion

#define CreateMB         OS_CreateMB
#define ClearMB          OS_ClearMB
#define PutMail          OS_PutMail
#define PutMailCond      OS_PutMailCond
#define GetMail          OS_GetMail
#define GetMailCond      OS_GetMailCond
#define PutMail1         OS_PutMail1
#define PutMailCond1     OS_PutMailCond1
#define GetMail1         OS_GetMail1
#define GetMailCond1     OS_GetMailCond1
#define DeleteMB         OS_DeleteMB
#define GetMsgCnt        OS_GetMsgCnt
#define GetMessageCnt    OS_GetMsgCnt
#define OS_CREATEMB      OS_CreateMB

#define TASK    OS_TASK
#define TIMER   OS_TIMER
#define RSEMA   OS_RSEMA
#define CSEMA   OS_CSEMA
#define MAILBOX OS_MAILBOX

#define SDI()    OS_IncDI()
#define RI()     OS_DecRI()
#define di()     OS_DI()
#define ri()     OS_RestoreI()
#define OS_di()	 OS_DI()
#define OS_ri()	 OS_RestoreI()
#define OS_RI()  OS_DecRI()
#define OS_SDI() OS_IncDI()
#define OS_DisableI()    { OS_DI(); }
#define OS_EnableI()     { OS_EI(); }

#define RTOS_Error  OS_Error
#define OS_GoHalt   OS_Idle
#define pCurrentTask OS_pCurrentTask


/* Compatibility with pre V2.72 versions */

#define OS_TimeMS OS_GetTime()
#define    TimeMS OS_GetTime()

/* Compatibility with pre V3.00 versions */
#define OS_STAT_GetMaxLatency(pTask) 0
#define OS_GetMsgCnt OS_GetMessageCnt 

#endif

/* Compatibility with manual */
#define OS_CREATEMB          OS_CreateMB
#define OS_GetUseCnt         OS_GetSemaValue     
#define OS_WaitCSema_Timed   OS_WaitCSemaTimed
#define OS_WaitEvent_Timed	 OS_WaitEventTimed


#ifdef __cplusplus
  }
#endif


#endif /* RTOS_H_INCLUDED */

